"use strict";
/**
 * @fileOverview excel2json main
 * @name index
 * @author Yuhei Aihara
 * https://github.com/iyu/excel2json
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = __importDefault(require("lodash"));
var parser_1 = __importDefault(require("./excel/parser"));
var logger_1 = __importDefault(require("./logger"));
var Excel2Json = /** @class */ (function () {
    function Excel2Json() {
        var _this = this;
        this.opts = {
            // Cell with a custom sheet option.
            option_cell: 'A1',
            // Line with a data attribute.
            attr_line: 2,
            // Line with a data.
            data_line: 4,
            // ref key
            ref_key: '_id',
            // Custom logger.
            logger: undefined,
        };
        this.logger = logger_1.default;
        this._parser = {
            number: function (d) {
                if (d.length >= 18) {
                    // IEEE754
                    return Number(Number(d).toFixed(8));
                }
                return Number(d);
            },
            num: function (d) {
                return _this._parser.number(d);
            },
            boolean: function (d) {
                return !!d && d.toLowerCase() !== 'false' && d !== '0';
            },
            bool: function (d) {
                return _this._parser.boolean(d);
            },
            date: function (d) {
                return Math.round((((Number(d) - 25569) * 24)
                    + (new Date().getTimezoneOffset() / 60)) * 3600000);
            },
            auto: function (d) {
                return Number.isFinite(Number(d)) ? d : _this._parser.number(d);
            },
        };
    }
    /**
     * setup
     * @param {Object} options
     * @example
     * var options = {
     *     option_cell: 'A1',
     *     attr_line: 2,
     *     data_line: 4,
     *     ref_key: '_id',
     *     logger: CustomLogger
     * };
     */
    Excel2Json.prototype.setup = function (options) {
        lodash_1.default.assign(this.opts, options);
        if (this.opts.logger) {
            this.logger = this.opts.logger;
        }
        this.logger.info('excel2json setup');
        return this;
    };
    /**
     * format
     * @param {Array} cells
     * @private
     * @example
     * var cells = [
     *     { cell: 'A1', value: '{}' }, { cell: 'A4', value: '_id' },,,
     * ]
     */
    Excel2Json.prototype._format = function (cells) {
        var _this = this;
        var beforeRow;
        var idx = {};
        var list = [];
        var opts = {
            attr_line: this.opts.attr_line,
            data_line: this.opts.data_line,
            ref_key: this.opts.ref_key,
            format: undefined,
        };
        lodash_1.default.forEach(cells, function (cell) {
            if (beforeRow !== cell.row) {
                lodash_1.default.forEach(idx, function (i) {
                    if (i.type !== 'format') {
                        i.value += 1;
                    }
                });
                beforeRow = cell.row;
            }
            if (cell.cell === _this.opts.option_cell) {
                var _opts = JSON.parse(cell.value);
                lodash_1.default.assign(opts, _opts);
                return;
            }
            if (cell.row === opts.attr_line) {
                var type = cell.value.match(/:(\w+)$/);
                var key = cell.value.replace(/:\w+$/, '');
                var keys = key.split('.');
                opts.format = opts.format || {};
                opts.format[cell.column] = {
                    type: type && type[1],
                    key: key,
                    keys: keys,
                };
                return;
            }
            var format = opts.format && opts.format[cell.column];
            var data;
            var _idx;
            if (cell.row < opts.data_line || !format) {
                return;
            }
            if (format.type && format.type.toLowerCase() === 'index') {
                _idx = parseInt(cell.value, 10);
                if (!idx[format.key] || idx[format.key].value !== _idx) {
                    idx[format.key] = {
                        type: 'format',
                        value: _idx,
                    };
                    lodash_1.default.forEach(idx, function (i, key) {
                        if (new RegExp("^" + format.key + ".+$").test(key)) {
                            idx[key].value = 0;
                        }
                    });
                }
                return;
            }
            if (format.key === opts.ref_key || format.key === '__ref') {
                idx = {};
                list.push({});
            }
            data = lodash_1.default.last(list);
            lodash_1.default.forEach(format.keys, function (_key, i) {
                var isArray = /^#/.test(_key);
                var isSplitArray = /^\$/.test(_key);
                var __key;
                if (isArray) {
                    _key = _key.replace(/^#/, '');
                    data[_key] = data[_key] || [];
                }
                if (isSplitArray) {
                    _key = _key.replace(/^\$/, '');
                }
                if (i + 1 !== format.keys.length) {
                    if (isArray) {
                        __key = format.keys.slice(0, i + 1).join('.');
                        _idx = idx[__key];
                        if (!_idx) {
                            idx[__key] = {
                                type: 'normal',
                                value: data[_key].length ? data[_key].length - 1 : 0,
                            };
                            _idx = idx[__key];
                        }
                        data[_key][_idx.value] = data[_key][_idx.value] || {};
                        data = data[_key][_idx.value];
                        return;
                    }
                    data[_key] = data[_key] || {};
                    data = data[_key];
                    return;
                }
                if (isArray) {
                    __key = format.keys.slice(0, i + 1).join('.');
                    _idx = idx[__key];
                    if (!_idx) {
                        idx[__key] = {
                            type: 'normal',
                            value: data[_key].length ? data[_key].length - 1 : 0,
                        };
                        _idx = idx[__key];
                    }
                    data = data[_key];
                    _key = _idx.value;
                }
                if (data[_key]) {
                    return;
                }
                var type = format.type && format.type.toLowerCase();
                if (_this._parser[type]) {
                    data[_key] = isSplitArray ? cell.value.split(',').map(_this._parser[type]) : _this._parser[type](cell.value);
                }
                else {
                    data[_key] = isSplitArray ? cell.value.split(',') : cell.value;
                }
            });
        });
        return {
            opts: opts,
            list: list,
        };
    };
    /**
     * find origin data
     * @param dataMap
     * @param opts
     * @param data
     * @private
     */
    Excel2Json.prototype._findOrigin = function (dataMap, opts, data) {
        var origin = dataMap[data.__ref];
        if (!origin || !opts.key) {
            this.logger.error('not found origin.', JSON.stringify(data));
            return;
        }
        var keys = opts.key.split('.');
        var __in = data.__in ? data.__in.split('.') : [];
        for (var i = 0; i < keys.length; i++) {
            if (/^#/.test(keys[i])) {
                var key = keys[i].replace(/^#/, '');
                var index = __in[i] && __in[i].replace(/^#.+:(\d+)$/, '$1');
                if (!index) {
                    this.logger.error('not found index.', JSON.stringify(data));
                    return;
                }
                origin[key] = origin[key] || [];
                origin = origin[key];
                origin[index] = origin[index] || {};
                origin = origin[index];
            }
            else if (keys[i] === '$') {
                origin = origin[__in[i]];
            }
            else if (i + 1 === keys.length) {
                origin[keys[i]] = origin[keys[i]] || (opts.type === 'array' ? [] : {});
                origin = origin[keys[i]];
            }
            else {
                origin[keys[i]] = origin[keys[i]] || {};
                origin = origin[keys[i]];
            }
            if (!origin) {
                this.logger.error('not found origin parts.', JSON.stringify(data));
                return;
            }
        }
        if (opts.type === 'array') {
            if (!Array.isArray(origin)) {
                this.logger.error('is not Array.', JSON.stringify(data));
                return;
            }
            origin.push({});
            origin = origin[origin.length - 1];
        }
        else if (opts.type === 'map') {
            if (!data.__key) {
                this.logger.error('not found __key.', JSON.stringify(data));
                return;
            }
            origin[data.__key] = {};
            origin = origin[data.__key];
        }
        return origin;
    };
    /**
     * excel parser main script
     * @param {String} filepath
     * @param {Array} sheets
     * @param {Function} callback
     */
    Excel2Json.prototype.parse = function (filepath, sheets, callback) {
        return __awaiter(this, void 0, void 0, function () {
            var excelData, e_1, errList, promises, result, e_2;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, parser_1.default.execute(filepath, sheets)];
                    case 1:
                        excelData = _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        e_1 = _a.sent();
                        return [2 /*return*/, callback(e_1)];
                    case 3:
                        promises = lodash_1.default.map(excelData, function (sheetData) {
                            var result;
                            try {
                                result = _this._format(sheetData.cells);
                            }
                            catch (e) {
                                _this.logger.error('invalid sheet format.', sheetData.num, sheetData.name);
                                errList = errList || [];
                                errList.push({
                                    num: sheetData.num,
                                    name: sheetData.name,
                                    error: e,
                                });
                                return Promise.resolve();
                            }
                            return new Promise(function (resolve) {
                                setImmediate(function () {
                                    resolve({
                                        num: sheetData.num,
                                        name: sheetData.name,
                                        opts: result.opts,
                                        list: result.list,
                                    });
                                });
                            });
                        });
                        _a.label = 4;
                    case 4:
                        _a.trys.push([4, 6, , 7]);
                        return [4 /*yield*/, Promise.all(promises)];
                    case 5:
                        result = _a.sent();
                        return [3 /*break*/, 7];
                    case 6:
                        e_2 = _a.sent();
                        callback(e_2);
                        return [2 /*return*/];
                    case 7:
                        callback(null, lodash_1.default.compact(result), errList);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * sheetDatas to json
     * @param {Array} sheetDatas
     * @param {Function} callback
     */
    Excel2Json.prototype.toJson = function (sheetDatas, callback) {
        var collectionMap = {};
        var optionMap = {};
        var errors = {};
        for (var i = 0; i < sheetDatas.length; i++) {
            var sheetData = sheetDatas[i];
            var opts = sheetData.opts;
            var name_1 = opts.name || sheetData.name;
            var refKey = opts.ref_key;
            collectionMap[name_1] = collectionMap[name_1] || {};
            var dataMap = collectionMap[name_1];
            if (!optionMap[name_1]) {
                optionMap[name_1] = opts;
            }
            else {
                optionMap[name_1] = lodash_1.default.assign({}, opts, optionMap[name_1]);
                lodash_1.default.assign(optionMap[name_1].format, opts.format);
            }
            for (var j = 0; j < sheetData.list.length; j++) {
                var data = sheetData.list[j];
                if (!opts.type || opts.type === 'origin') {
                    dataMap[data[refKey]] = data;
                }
                else {
                    var origin_1 = this._findOrigin(dataMap, opts, data);
                    if (origin_1) {
                        delete data.__ref;
                        delete data.__in;
                        delete data.__key;
                        lodash_1.default.assign(origin_1, data);
                    }
                    else {
                        errors[name_1] = errors[name_1] || [];
                        errors[name_1].push(data);
                    }
                }
            }
        }
        callback(lodash_1.default.isEmpty(errors) ? null : errors, collectionMap, optionMap);
    };
    return Excel2Json;
}());
exports.default = new Excel2Json();
